#include "IRImagerHandler.h"
#include <string.h>
#include <iostream>
#include <sstream>

using namespace std;

IRImagerHandler::IRImagerHandler(IRImager* imager)
{
  _imager    = imager;
  _imager->setClient(this);
  _thermal   = NULL;
  _yuyv      = NULL;

  _iBuilder.setPaletteScalingMethod(evo::eMinMax);
  _iBuilder.setPalette(evo::eIron);
}

IRImagerHandler::~IRImagerHandler()
{
  if (_thermal) delete[] _thermal;
  if (_yuyv)    delete[] _yuyv;
}

void IRImagerHandler::run()
{
	while (true)
	{		
		//get the last metadata with pif input values
		const IRFrameMetadata irFrameMetadata = _imager->getLastMetadata();
		stringstream ss;

		//Print out digital input values
		ss << "PIF Polling values" << endl;
		ss << "DIs: ";
		for (vector<bool>::const_iterator it = irFrameMetadata.pifDIs.begin();
			it != irFrameMetadata.pifDIs.end();
			it++)
		{
			ss << *it << " ";
		}

		//Print out analog input values
		ss << endl << "AIs: ";
		for (vector<float>::const_iterator it = irFrameMetadata.pifAIs.begin();
			it != irFrameMetadata.pifAIs.end();
			it++)
		{
			ss << *it << " ";
		}

		ss << endl;
		ss << "PIF Polling end" << endl;
		cout << ss.rdbuf();
		Sleep(100);
	}
}

void IRImagerHandler::displayEvents(std::vector<IREventData> events) {
	for (vector<IREventData>::iterator it = events.begin();
		it != events.end();
		it++)
	{
		cout << "\tChannel: " << +it->channel << " Type: ";
		switch (it->inputType)
		{
		case  IREventInputType::DigitalInput:
			cout << "DigitalInput";
			break;
		case  IREventInputType::AnalogInput:
			cout << "AnalogInput";
			break;
		case  IREventInputType::Software:
			cout << "Software";
			break;
		default:
			cout << "Unknown";
			break;
		}
		cout << endl;
	}
}

void IRImagerHandler::onThermalFrame(unsigned short* thermal, unsigned int w, unsigned int h, IRFrameMetadata meta, void* arg)
{
	//Access to thermal frame corresponding pif input values over meta data:

	stringstream ss;
	ss << "PIF Frameid " << meta.counter << " values" << endl;
	//Print out digital input values
	ss << "DIs: ";
	for (vector<bool>::const_iterator it = meta.pifDIs.begin();
		it != meta.pifDIs.end();
		it++)
	{
		ss << *it << " ";
	}

	//Print out analog input values
	ss << endl << "AIs: ";
	for (vector<float>::const_iterator it = meta.pifAIs.begin();
		it != meta.pifAIs.end();
		it++)
	{
		ss << *it << " ";
	}

	ss << endl;
	ss << "PIF Frameid " << meta.counter << " end" << endl;
	cout << ss.rdbuf();

	//Set do[0] to true, if ai[0] > 4V, else false
	if (meta.pifAIs.size() > 0){
		_imager->setPifDO(0, meta.pifAIs[0] > 4);
	}
}

/**
* Overwritten method from IRImagerClient
*/
void IRImagerHandler::onThermalFrameEvent(unsigned short* thermal, unsigned short* energy, unsigned int w, unsigned int h, IRFrameMetadata meta, std::vector<IREventData> events, void* arg)
{
	cout << "onThermalFrameEvent" << endl;
	displayEvents(events);
}

void IRImagerHandler::onVisibleFrame(unsigned char* yuyv, unsigned int w, unsigned int h, IRFrameMetadata meta, void* arg)
{
  if (_yuyv == NULL) _yuyv = new unsigned char[w*h * 2];
  memcpy(_yuyv, yuyv, 2 * w*h*sizeof(*yuyv));
}

/**
* @brief will be called on pif snapshot event.
* @param events List of triggered events.
*/

void IRImagerHandler::onVisibleFrameEvent(unsigned char *data, unsigned int w, unsigned int h, IRFrameMetadata meta, std::vector<IREventData> events, void *arg)
{
	cout << "onVisibleFrameEvent" << endl;
	displayEvents(events);
}

void IRImagerHandler::onFlagStateChange(evo::EnumFlagState fs, void* arg)
{
  cout << "Flag state: " << fs << endl;
}

void IRImagerHandler::onRawFrame(unsigned char* data, int size)
{
  double fps;
  if (_frc.trigger(&fps))
    cout << "Frame rate: " << fps << " fps" << endl;
  _imager->process(data);
}

void IRImagerHandler::onProcessExit(void* arg)
{

}